/*
 * Copyright 2009 Red Hat, Inc.
 * Red Hat licenses this file to you under the Apache License, version
 * 2.0 (the "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *    http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.  See the License for the specific language governing
 * permissions and limitations under the License.
 */
package org.hornetq.jms.example;

import java.io.IOException;
import java.net.MalformedURLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.hornetq.api.core.management.MessageCounterInfo;
import org.hornetq.api.core.management.ObjectNameBuilder;
import org.hornetq.api.jms.management.JMSQueueControl;

import org.hornetq.common.example.HornetQExample;

/**
 * This sample is supposed to show how concurrent consumers are not buffering messages. the
 * assumption is that if there is no buffering than after each consumer handles it's first message
 * the next messages are supposed to be received ordered.
 * 
 * Each consumer adds to a concurrent queue the message it currently handles and when all the
 * consumers finished their work a check is done to make sure that all the items in the queue are
 * sorted - if this is not the case than the test fails
 * 
 * This test is based on the test org.hornetq.jms.example.NoConsumerBufferingExample
 * 
 * @author Noa Drach
 */
public class NoConcurrentConsumersBufferingExample extends HornetQExample {
	private static final String JMX_URL = "service:jmx:rmi:///jndi/rmi://localhost:3001/jmxrmi";
	private final int messageNum;
	private final int consumersNum;
	public final int minDelay;
	public final int maxDelay;
	private final ConcurrentLinkedQueue<Integer> messagesReceived = new ConcurrentLinkedQueue<Integer>();

	private boolean isRunning = true;

	public NoConcurrentConsumersBufferingExample(int messageNum, int consumersNum, int minDelay,
			int maxDelay) {
		this.messageNum = messageNum;
		this.consumersNum = consumersNum;
		this.minDelay = minDelay;
		this.maxDelay = maxDelay;
	}

	public static void main(final String[] args) {

		int messageNum = args != null && args.length >= 1 ? Integer.parseInt(args[0]) : 100;
		int consumersNum = args != null && args.length >= 2 ? Integer.parseInt(args[1]) : 10;
		int minDelay = args != null && args.length >= 3 ? Integer.parseInt(args[2]) : 1;
		int maxDelay = args != null && args.length >= 4 ? Integer.parseInt(args[3]) : 120;

		new NoConcurrentConsumersBufferingExample(messageNum, consumersNum, minDelay, maxDelay)
				.run(args);

	}

	@Override
	public boolean runExample() throws Exception {

		// Create messages
		MessagingHandler messagingHandler = new MessagingHandler("localhost", "5445");

		for (int i = 0; i < messageNum; i++) {
			messagingHandler.sendMessage(i + 1);
		}

		// close the connection used to create the sender
		messagingHandler.cleanConnections();

		// Consume the messages
		LinkedList<Consumer> consumers = new LinkedList<Consumer>();

		JMSQueueControl queueControl = getQueueControl();

		printMessege("**************************  Messages In queue " + messageCount(queueControl));
		for (int i = 0; i < consumersNum; i++) {

			Consumer consumer = new Consumer(i + 1, this);
			consumer.start();
			consumers.add(consumer);

		}

		long lastPrint = System.currentTimeMillis();

		do {

			// Print a message every 15 seconds
			if (System.currentTimeMillis() - lastPrint >= 15000) {

				printMessege("**************************  Messages In queue "
						+ messageCount(queueControl));
				lastPrint = System.currentTimeMillis();
			}

		} while (messageCount(queueControl) > 0);

		isRunning = false;

		// Wait for all the consumers to stop
		for (Consumer consumer : consumers) {
			if (consumer.isAlive()) {
				consumer.join();
			}
		}

		// Check the test results

		boolean result = true;
		Integer prev = messagesReceived.poll();
		Integer poll = messagesReceived.poll();
		while (poll != null) {

			System.out.println(poll + ">" + prev + " ? " + (poll > prev));
			if (poll < prev) {

				System.out.println("Messages recieved out of sequance - test failure");
				result = false;
			}
			prev = poll;
			poll = messagesReceived.poll();
		}

		return result;
	}

	private long messageCount(JMSQueueControl queueControl) throws Exception {

		String counters = queueControl.listMessageCounter();
		MessageCounterInfo messageCounter = MessageCounterInfo.fromJSON(counters);
		return messageCounter.getDepth();
	}

	private JMSQueueControl getQueueControl() throws Exception, IOException, MalformedURLException {
		ObjectName on = ObjectNameBuilder.DEFAULT.getJMSQueueObjectName("exampleQueue");
		JMXConnector connector = JMXConnectorFactory.connect(new JMXServiceURL(JMX_URL),
				new HashMap());
		MBeanServerConnection mbsc = connector.getMBeanServerConnection();
		JMSQueueControl queueControl = MBeanServerInvocationHandler.newProxyInstance(mbsc, on,
				JMSQueueControl.class, false);
		return queueControl;
	}

	public void printMessege(String message) {
		SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss,S");
		System.out.println(dateFormat.format(new Date()) + "\t" + message);

	}

	public void printError(String message) {
		SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss,S");
		System.err.println(dateFormat.format(new Date()) + "\t" + message);

	}

	public boolean isRunning() {
		return isRunning;
	}

	public void addMessage(String message) {

		messagesReceived.add(Integer.parseInt(message.split("#")[1]));
	}

}
