package org.hornetq.jms.example;

import java.util.HashMap;
import java.util.Map;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.jms.HornetQJMSClient;
import org.hornetq.api.jms.JMSFactoryType;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;
import org.hornetq.core.remoting.impl.netty.TransportConstants;
import org.hornetq.jms.client.HornetQConnectionFactory;

public class MessagingHandler {

	private static final int SLEEP_BETWEEN_CONNECTION_TRIES_INTERVAL_LENGTH = 30000;
	private static final int MAX_CONNECTION_FAILURES_ALLOWED = 10;

	private final String serverName;
	private final String serverPort;

	private TransportConfiguration transportConfiguration;
	private HornetQConnectionFactory connectionFactory;

	private Queue queue;

	private QueueConnection queueConn;
	private QueueSession queueSession;
	private QueueReceiver queueReceiver;
	private QueueSender queueProducer;

	public MessagingHandler(String serverName, String serverPort) {
		this.serverName = serverName;
		this.serverPort = serverPort;

		setupMessagingConnection();

		initConnections();

	}

	private void setupMessagingConnection() {
		final Map<String, Object> p = new HashMap<String, Object>();
		p.put(TransportConstants.HOST_PROP_NAME, serverName);
		p.put(TransportConstants.PORT_PROP_NAME, serverPort);

		transportConfiguration = new TransportConfiguration(NettyConnectorFactory.class.getName(),
				p);
		connectionFactory = HornetQJMSClient.createConnectionFactoryWithoutHA(
				JMSFactoryType.QUEUE_CF, transportConfiguration);

		connectionFactory.setConnectionTTL(-1);
		connectionFactory.setConsumerWindowSize(0);

	}

	/**
	 * This method initializes all the objects needed to connect to the JBoss queues
	 */
	private void initConnections() {
		try {

			// Try to contact the missions queue for the first time
			makeInitialContact();

			// create a queue connection
			queueConn = connectionFactory.createQueueConnection();

			// create a queue session - the session is transacted
			queueSession = queueConn.createQueueSession(true, Session.AUTO_ACKNOWLEDGE);

			// create a queue receiver and producer
			queueReceiver = queueSession.createReceiver(queue);
			queueProducer = queueSession.createSender(queue);
			queueProducer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

			// start the connection

			queueConn.start();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void makeInitialContact() throws Exception {
		boolean connectionFailed = true;
		long connectionFailureCount = 0;

		// if at first you fail, try and try again
		while (connectionFailed) {
			try {

				queue = HornetQJMSClient.createQueue("exampleQueue");
				connectionFailed = false;
			} catch (Exception e) {

				// if the connection failed
				connectionFailureCount++;
				e.printStackTrace();
				// sleep for 30 seconds and try again
				Thread.sleep(SLEEP_BETWEEN_CONNECTION_TRIES_INTERVAL_LENGTH);
			}
		}

		if (connectionFailureCount == MAX_CONNECTION_FAILURES_ALLOWED) {
			throw new Exception("Failed to connect to the JBoss server. "
					+ MAX_CONNECTION_FAILURES_ALLOWED
					* SLEEP_BETWEEN_CONNECTION_TRIES_INTERVAL_LENGTH / 1000 / 60
					+ " minute timeout exceeded! Please verify that the server is running.");
		}
	}

	/**
	 * This method clears the old connections to JBoss and recreates them.
	 */
	public void refreshConnection() {
		cleanConnections();
		initConnections();
	}

	/**
	 * This method disconnects and clears all the objects used to connect to the JBoss queues
	 */
	public void cleanConnections() {
		try {

			try {
				if (queueConn != null) {
					queueConn.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

			try {
				if (queueSession != null) {
					queueSession.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				if (queueReceiver != null) {
					queueReceiver.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

			try {
				if (queueProducer != null) {
					queueProducer.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

		} finally {
			queueProducer = null;
			queueReceiver = null;
			queueSession = null;
			queueConn = null;
		}

	}

	public QueueReceiver getQueueReceiver() {
		return queueReceiver;
	}

	public Message getMessage() throws JMSException {
		return getQueueReceiver().receiveNoWait();
	}

	public void sendMessage(int num) throws JMSException {
		TextMessage res = queueSession.createTextMessage("Message #" + num);

		queueProducer.send(res);
		queueSession.commit();

	}

	public void commit() throws JMSException {
		queueSession.commit();

	}

	public void rollback() throws JMSException {
		queueSession.rollback();
	}
}
